import {
  Button,
  Checkbox,
  Flex,
  Form,
  Modal,
  Result,
  Space,
  Typography,
  notification,
} from "antd";
import {
  Constants,
  EncodeStringToBase64,
  myFetch,
  setUserSessionToLocalStorage,
  showInputsInvalidNotification,
  showUnkownErrorNotification,
} from "../../utils";
import { useEffect, useState } from "react";
import {
  MyAccountNameFormInput,
  MyCompanyNameFormInput,
  MyPasswordFormInput,
  MyUsernameFormInput,
} from "../../Components/MyFormInputs";
import { useTranslation } from "react-i18next";
import MyAppLogo from "../../Components/MyAppLogo";
import { useNavigate } from "react-router-dom";

export const AuthenticationMethod = {
  LOGIN: 1,
  SIGNUP: 2,
};

export default function Authentication({ method }) {
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  return (
    <>
      {notificationContextHolder}

      <Modal
        open={true}
        mask={false}
        closable={false}
        centered
        keyboard={false}
        footer={null}
      >
        <div
          style={{
            display: "flex",
            justifyContent: "center",
          }}
        >
          <MyAppLogo height={80} />
        </div>

        {method === AuthenticationMethod.LOGIN ? (
          <Login notificationApi={notificationApi} />
        ) : (
          <SignUp notificationApi={notificationApi} />
        )}
      </Modal>
    </>
  );
}

function RememberMeCheckbox() {
  const { t } = useTranslation();

  return (
    <Form.Item name="rememberMe" valuePropName="checked">
      <Checkbox>{t("authentication.rememberMe")}</Checkbox>
    </Form.Item>
  );
}

const LoginStep = {
  ACCOUNT_NAME: 1,
  PASSWORD: 2,
  PENDING_DELETION: 3,
  INIT_LOGIN: 4,
  BANNED: 5,
  _BACK_TO_PASSWORD: 6, // needed for going back from pending deletion to password step as the useEffect for the account name is triggered and would set the step to account name
};

// First step: account name -> get state of account by backend
// Second step: if account is active -> password -> login otherwise show the state of the account (e. g. deleted, banned)
function Login({ notificationApi }) {
  const { t } = useTranslation();
  const navigate = useNavigate();

  const [isRequesting, setIsRequesting] = useState(false);
  const [step, setStep] = useState(LoginStep.ACCOUNT_NAME);

  const [form] = Form.useForm();

  const accountName = Form.useWatch("accountName", form);

  const showErrorNotification = (errStatus) => {
    if (errStatus === 400) {
      notificationApi["error"]({
        message: t("authentication.login.request.400.title"),
        description: t("authentication.login.request.400.description"),
      });
    }
  };

  useEffect(() => {
    if (!accountName) return;

    // triggered if step was set to pending deletion and the user wants to reactivate the account
    if (step === LoginStep._BACK_TO_PASSWORD) {
      setStep(LoginStep.PASSWORD);
      return;
    }

    // reset step if account name changed for handling the state of the account
    if (step === LoginStep.PASSWORD || step === LoginStep.INIT_LOGIN) {
      setStep(LoginStep.ACCOUNT_NAME);
    }
  }, [accountName]);

  if (step === LoginStep.PENDING_DELETION) {
    return (
      <Result
        status="warning"
        title={t("authentication.login.stateAccountPendingDeletion.title")}
        subTitle={t(
          "authentication.login.stateAccountPendingDeletion.description"
        )}
        extra={[
          <Button key={0} onClick={() => setStep(LoginStep.ACCOUNT_NAME)}>
            {t("common.button.cancel")}
          </Button>,
          <Button
            key={1}
            type="primary"
            onClick={() => {
              console.log("reactivate account");
              setStep(LoginStep._BACK_TO_PASSWORD);
            }}
          >
            {t("authentication.login.stateAccountPendingDeletion.button")}
          </Button>,
        ]}
      />
    );
  }

  if (step === LoginStep.BANNED) {
    return (
      <Result
        status="error"
        title={t("authentication.login.stateAccountBanned.title")}
        subTitle={
          <Space direction="vertical">
            <Typography.Text type="secondary">
              {t("authentication.login.stateAccountBanned.infoSupport")}
            </Typography.Text>

            <Button type="link" href={`mailto:${Constants.SUPPORT_EMAIL}`}>
              {Constants.SUPPORT_EMAIL}
            </Button>
          </Space>
        }
        extra={[
          <Button
            key={0}
            onClick={() => {
              setStep(LoginStep.ACCOUNT_NAME);
            }}
          >
            {t("authentication.login.stateAccountBanned.backButton")}
          </Button>,
        ]}
      />
    );
  }

  return (
    <Form
      form={form}
      layout="vertical"
      requiredMark={false}
      initialValues={{
        rememberMe: false,
      }}
    >
      {step === LoginStep.INIT_LOGIN && (
        <Typography.Paragraph type="secondary">
          {t("authentication.login.stateInitLogin.info")}
        </Typography.Paragraph>
      )}

      <MyAccountNameFormInput
        disableAccountNameCheck={true}
        hasFeedback={false}
      />

      <div
        style={{
          display:
            step === LoginStep.PASSWORD || step === LoginStep.INIT_LOGIN
              ? "block"
              : "none",
        }}
      >
        <MyPasswordFormInput />

        <RememberMeCheckbox />
      </div>

      <Button
        type="primary"
        size="large"
        block
        loading={isRequesting}
        htmlType="submit"
        onClick={() => {
          let validateFields = [];

          if (step === LoginStep.ACCOUNT_NAME) {
            validateFields = ["accountName"];
          } else {
            validateFields = ["accountName", "password", "rememberMe"];
          }

          form
            .validateFields(validateFields)
            .then((values) => {
              setIsRequesting(true);

              let body = {
                accountName: values.accountName.toLocaleLowerCase(),
              };

              if (
                step === LoginStep.PASSWORD ||
                step === LoginStep.INIT_LOGIN
              ) {
                body.password = EncodeStringToBase64(values.password);
                body.rememberMe = values.rememberMe;
              }

              myFetch({
                url: `/user/auth/login`,
                method: "POST",
                body: body,
                notificationApi: notificationApi,
                t: t,
              })
                .then((data) => {
                  setIsRequesting(false);

                  if (
                    step === LoginStep.PASSWORD ||
                    step === LoginStep.INIT_LOGIN
                  ) {
                    setUserSessionToLocalStorage(data.XAuthorization);
                    window.location.href = "/";
                    return;
                  }

                  if (data.state === undefined) return;

                  switch (data.state) {
                    case Constants.ACCOUNT_STATE.ACTIVE:
                      setStep(LoginStep.PASSWORD);
                      break;
                    case Constants.ACCOUNT_STATE.PENDING_DELETION:
                      setStep(LoginStep.PENDING_DELETION);
                      break;
                    case Constants.ACCOUNT_STATE.BANNED:
                      setStep(LoginStep.BANNED);
                      break;
                    case Constants.ACCOUNT_STATE.INIT_LOGGING:
                      setStep(LoginStep.INIT_LOGIN);
                      break;
                    default:
                      showUnkownErrorNotification(notificationApi, t);
                      break;
                  }
                })
                .catch((errStatus) => {
                  showErrorNotification(errStatus);
                  setIsRequesting(false);
                });
            })
            .catch(() => showInputsInvalidNotification(notificationApi, t));
        }}
      >
        {t("authentication.login.button")}
      </Button>

      <Flex justify="center" style={{ paddingTop: 12 }}>
        <Typography.Text>
          {t("authentication.login.dontHaveAccount")}{" "}
          <Button
            type="link"
            style={{ padding: 0 }}
            onClick={() =>
              navigate(Constants.ROUTE_PATHS.AUTHENTICATION.SIGN_UP)
            }
          >
            {t("authentication.signUpLink")}
          </Button>
        </Typography.Text>
      </Flex>
    </Form>
  );
}

function SignUp({ notificationApi }) {
  const { t, i18n } = useTranslation();
  const navigate = useNavigate();

  const [isRequesting, setIsRequesting] = useState(false);

  const [form] = Form.useForm();

  const showErrorNotification = (errStatus) => {
    if (errStatus === 400) {
      notificationApi["error"]({
        message: t("authentication.signUp.request.400.title"),
        description: t("authentication.signUp.request.400.description"),
      });
    }
  };

  const handleSubmit = () => {
    form
      .validateFields()
      .then((values) => {
        setIsRequesting(true);

        let body = {
          companyName: values.companyName,
          accountName: values.accountName.toLocaleLowerCase(),
          password: EncodeStringToBase64(values.password),
          username: values.username,
          language: i18n.language,
          rememberMe: values.rememberMe,
        };

        myFetch({
          url: `/user/auth/signup`,
          method: "POST",
          body: body,
          notificationApi: notificationApi,
          t: t,
        })
          .then((data) => {
            setUserSessionToLocalStorage(data.XAuthorization);
            window.location.href = "/";
          })
          .catch((errStatus) => {
            showErrorNotification(errStatus);
            setIsRequesting(false);
          });
      })
      .catch((err) => {
        console.log(err);
        showInputsInvalidNotification(notificationApi, t);
      });
  };

  return (
    <Form
      form={form}
      layout="vertical"
      requiredMark={false}
      initialValues={{
        rememberMe: false,
      }}
    >
      <MyCompanyNameFormInput />

      <MyUsernameFormInput
        propsFormItem={{ extra: t("common.usernameInfo") }}
      />

      <MyAccountNameFormInput
        hasFeedback={true}
        propsFormItem={{ extra: t("common.accountNameInfo") }}
      />

      <MyPasswordFormInput />

      <RememberMeCheckbox />

      <Button
        type="primary"
        size="large"
        block
        onClick={handleSubmit}
        loading={isRequesting}
      >
        {t("authentication.signUp.button")}
      </Button>

      <Flex justify="center" style={{ paddingTop: 12 }}>
        <Typography.Text>
          {t("authentication.signUp.alreadyHaveAccount")}{" "}
          <Button
            type="link"
            style={{ padding: 0 }}
            onClick={() => navigate(Constants.ROUTE_PATHS.AUTHENTICATION.LOGIN)}
          >
            {t("authentication.loginLink")}
          </Button>
        </Typography.Text>
      </Flex>
    </Form>
  );
}
